--
-- Based on the following works:
-- https://github.com/MagicMau/ProceduralNameGenerator
-- https://github.com/Tw1ddle/MarkovNameGenerator
--

-- if os.getenv("LOCAL_LUA_DEBUGGER_VSCODE") == "1" then
--     require("lldebugger").start()
-- end
-- io.stdout:setvbuf("no")

local g = require("modules.generator")

function love.load()
    math.randomseed(os.clock())

    local trainingData = {}

    for line in love.filesystem.lines("names.txt") do
        local name = string.lower(line)
        table.insert(trainingData, name)
    end

    -- Initialize the generator
    -- @param1 trainingData A table of names to analyze
    -- @param2 order Best described as a cohesion value
    -- -- smaller = more random / unlikely / unpronounceable names
    -- -- higher = names become similar/exact to training data
    -- @param3 A Dirichlet prior is used to add a constant probability
    -- -- that any letter may be picked as the next letter.
    -- -- This acts as an additive smoothing factor and adds a bit more "randomness"
    -- -- to the generated output.
    local generator = g.new(trainingData, 3, 0.001)

    local count = 0
    while count < 10 do
        local found = false

        -- Generate a name
        local x = string.gsub(generator:generate(), "#", "")

        -- Optional:
        -- reject names found in training data
        -- reject short names
        -- reject long names
        for _, value in pairs(trainingData) do
            if x == value or #x < 4 or #x > 7 then
                found = true
                break
            end
        end

        if not found then
            count = count + 1
            print((x:gsub("^%l", string.upper)))
        end
    end
end
