local HS = require("modules.hashset")
local M  = require("modules.model")

local generator   = {}
generator.__index = generator

function generator.new(trainingData, order, smoothing)
    local instance = setmetatable({}, generator)

    instance.order     = order
    instance.smoothing = smoothing
    instance.models    = {}

    local alphabet = generator.getAlphabet(trainingData)
    for i = 1, order do
        instance.models[#instance.models + 1] =
        M.new(
            trainingData,
            order - (i - 1),
            smoothing,
            alphabet
        )
    end
    return instance
end

function generator.getAlphabet(trainingData)
    local letters = HS.new()
    letters:Add("#")
    for i = #trainingData, 1, -1 do
        local name = string.lower(trainingData[i])
        for j = 1, #name do
            local char = string.sub(name, j, j)
            if not string.find(" '-", char) then
                letters:Add(char)
            else
                trainingData[i] = nil
                break
            end
        end
    end
    return letters
end

function generator:generate()
    local name = string.rep("#", self.order)
    local letter = self:getLetter(name)
    while letter and letter ~= "#" do
        name = name .. letter;
        letter = self:getLetter(name);
    end
    return name
end

function generator:getLetter(name)
    local letter = nil
    local context = string.sub(name, #name - self.order + 1)
    for _, model in ipairs(self.models) do
        letter = model:generate(context)
        if not letter then
            context = string.sub(context, 2)
        else
            break
        end
    end
    return letter
end

return generator
