local HS = require("modules.hashset")
local insert = table.insert
--
--
local model = {}
model.__index = model

function model.new(trainingData, order, smoothing, alphabet)
    local instance = setmetatable({}, {
        __index = {
            train = model.train,
            retrain = model.retrain,
            generate = model.generate,
            selectIndex = model.selectIndex,
            buildChains = model.buildChains,
        }
    })

    instance.order        = order
    instance.smoothing    = smoothing
    instance.alphabet     = alphabet
    instance.observations = {}
    instance.chains       = {}

    instance:retrain(trainingData)
    return instance
end

function model:generate(context)
    if self.chains[context] then
        local chain = {}
        local temp = self.chains[context]
        for key, value in pairs(temp) do
            chain[#chain + 1] = { key = key, value = value[1] }
        end
        table.sort(chain, function(a, b)
            return a.key < b.key
        end)
        return self:selectIndex(chain)
    end
    return nil
end

function model:retrain(td)
    self:train(td)
    self:buildChains()
end

function model:train(td)
    for _, w in pairs(td) do
        local data = string.rep("#", self.order) .. w .. "#"
        for i = 1, #data - self.order do
            local key = string.sub(data, i, i - 1 + self.order)
            if not self.observations[key] then
                self.observations[key] = HS.new()
            end
            self.observations[key]:Add(string.sub(data, i + self.order, i + self.order))
        end
    end
end

function model:buildChains()
    self.chains = {}
    for context, _ in pairs(self.observations) do
        for prediction, _ in pairs(self.alphabet) do
            if not self.chains[context] then
                self.chains[context] = {}
            end
            if not self.chains[context][prediction] then
                self.chains[context][prediction] = {}
            end
            local count = 0
            local observation = self.observations[context]
            if observation then
                for key, value in pairs(observation) do
                    if key == prediction then count = value end
                end
                insert(self.chains[context][prediction], self.smoothing + count)
            end
        end
    end
end

function model:selectIndex(chain)
    local totals = {}
    local accumulator = 0

    for i, t in ipairs(chain) do
        accumulator = accumulator + t.value
        if not totals[i] then totals[i] = {} end
        totals[i] = { key = t.key, value = accumulator }
    end

    local rand = math.random() * accumulator

    for _, t in ipairs(totals) do
        if rand < t.value then
            return t.key
        end
    end
    return 1
end

return model
